import type { MaybeArray } from '@/utils/types';

export * from './zip';
export function create2DArray<T>(
  height: number,
  width: number,
  cell: T | ((x: number, y: number) => T),
): T[][] {
  return Array.from({ length: height }, (_, y) =>
    Array.from({ length: width }, (_, x) => (cell instanceof Function ? cell(x, y) : cell)),
  );
}
export function* iter2DArray<T>(
  grid: T[][],
): Generator<[x: number, y: number, item: T], void, unknown> {
  for (let y = 0; y < grid.length; y++) {
    for (let x = 0; x < grid[y].length; x++) {
      yield [x, y, grid[y][x]];
    }
  }
}
export function* entries<T>(it: Iterable<T>): Generator<[number, T], void, unknown> {
  let i = 0;
  for (const value of it) {
    yield [i++, value];
  }
}
export function get2DArrayItem<T>(
  grid: T[][],
  first: number,
  second: number,
  colFirst: boolean,
): T {
  return colFirst ? grid[first][second] : grid[second][first];
}
export function ensureArray<T>(obj: MaybeArray<T, true>): T[];
export function ensureArray(obj: unknown) {
  return Array.isArray(obj) ? obj : [obj];
}
export const arrayIncludes = <T, R>(array: T[], value: R): value is T & R =>
  array.includes(value as any);
export const len = (len: number) => ({ length: len });
