export type Future<T = void> = PromiseWithResolvers<T>;
export class FutureMap<K, V = void> extends Map<K, Future<V>> {
  add(key: K) {
    const future = Promise.withResolvers<V>();
    this.set(key, future);
    return future.promise;
  }

  resolve(key: K, value: V) {
    this.get(key)?.resolve(value);
    this.delete(key);
  }

  reject(key: K, reason: unknown) {
    this.get(key)?.reject(reason);
    this.delete(key);
  }
  waitAll() {
    return Promise.all(this.values().map((f) => f.promise));
  }
}
