import { entries } from './array';
export type Pos = [y: number, x: number];
export const toPos = (first: number, second: number, xFirst: boolean): Pos =>
  xFirst ? [second, first] : [first, second];
function key(first: number, second: number, xFirst: boolean) {
  const [y, x] = toPos(first, second, xFirst);
  return `${y}-${x}`;
}
export class TableView<T> {
  private map: Map<string, T>;
  size: Pos;
  constructor(size: Pos);
  constructor(tableView: TableView<T>);
  constructor(arg: Pos | TableView<T>) {
    if (arg instanceof TableView) {
      this.map = new Map(arg.map);
      this.size = [...arg.size];
    } else {
      this.map = new Map();
      this.size = [...arg];
    }
  }
  get filledSize() {
    return this.map.size;
  }
  get(first: number, second: number, xFirst: boolean = false) {
    return this.map.get(key(first, second, xFirst));
  }
  set(first: number, second: number, value: T, xFirst: boolean = false) {
    this.map.set(key(first, second, xFirst), value);
  }
  delete(first: number, second: number, xFirst: boolean = false) {
    this.map.delete(key(first, second, xFirst));
  }
  getLine(first: number, xFirst: boolean = false) {
    return new LineView(this, first, xFirst);
  }
  *iter(xFirst: boolean = false) {
    for (let i = 0; i < this.size[xFirst ? 1 : 0]; i++) {
      yield this.getLine(i, xFirst);
    }
  }
  toArray<R = T | undefined>(xFirst: boolean = false, converter?: (v: T | undefined) => R) {
    return Array.from(
      this.iter(xFirst).map((line) =>
        Iterator.from(line)
          .map((v) => (converter ? converter(v) : (v as unknown as R)))
          .toArray(),
      ),
    );
  }
  clear() {
    this.map.clear();
  }
  getEmptyPositions() {
    const emptyPositions: Pos[] = [];
    for (const [y, line] of entries(this.iter())) {
      for (const [x, tile] of entries(line)) {
        if (tile !== undefined) continue;
        emptyPositions.push([y, x]);
      }
    }
    return emptyPositions;
  }
}
export class LineView<T> {
  constructor(
    private table: TableView<T>,
    private readonly first: number,
    private readonly xFirst: boolean,
  ) {}
  get(second: number) {
    return this.table.get(this.first, second, this.xFirst);
  }
  set(second: number, value: T) {
    this.table.set(this.first, second, value, this.xFirst);
  }
  delete(second: number) {
    this.table.delete(this.first, second, this.xFirst);
  }
  get size() {
    return this.table.size[this.xFirst ? 0 : 1];
  }
  toArray() {
    return Array.from(this);
  }
  *[Symbol.iterator]() {
    for (let i = 0; i < this.size; i++) {
      yield this.get(i);
    }
  }
}
